/* libxsvg - Render SVG documents using the Xr library
 *
 * Copyright  2002 USC/Information Sciences Institute
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of
 * Information Sciences Institute not be used in advertising or
 * publicity pertaining to distribution of the software without
 * specific, written prior permission.  Information Sciences Institute
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * INFORMATION SCIENCES INSTITUTE DISCLAIMS ALL WARRANTIES WITH REGARD
 * TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL INFORMATION SCIENCES
 * INSTITUTE BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth <cworth@isi.edu>
 */

#include <stdlib.h>
#include <string.h>

#include "xsvgint.h"

xsvg_status_t
_xsvg_state_create (xsvg_state_t **state)
{
    *state = malloc (sizeof (xsvg_state_t));
    if (*state == NULL)
	return XSVG_STATUS_NO_MEMORY;

    _xsvg_state_init (*state);

    return XSVG_STATUS_SUCCESS;
}

xsvg_status_t
_xsvg_state_init (xsvg_state_t *state)
{
    /* trust libsvg to set all of these to reasonable defaults:
    state->fill_paint;
    state->stroke_paint;
    state->fill_opacity;
    state->stroke_opacity;
    */
    state->child_surface = NULL;

    state->font_family = strdup (XSVG_FONT_FAMILY_DEFAULT);
    if (state->font_family == NULL)
	return XSVG_STATUS_NO_MEMORY;

    state->font_size = 1.0;
    state->font_style = SVG_FONT_STYLE_NORMAL;
    state->font_weight = 400;
    state->font_dirty = 1;

    state->dash = NULL;
    state->num_dashes = 0;
    state->dash_offset = 0;

    state->opacity = 1.0;

    state->text_anchor = SVG_TEXT_ANCHOR_START;

    state->next = NULL;

    state->in_set_pattern_HACK = 0;

    return XSVG_STATUS_SUCCESS;
}

xsvg_status_t
_xsvg_state_init_copy (xsvg_state_t *state, const xsvg_state_t *other)
{
    _xsvg_state_deinit (state);

    if (other == NULL)
	return _xsvg_state_init (state);

    *state = *other;

    if (other->child_surface)
	state->child_surface = cairo_surface_create_similar (other->child_surface,
						      CAIRO_FORMAT_ARGB32,
						      state->viewport_width,
						      state->viewport_height);

    if (other->font_family)
	state->font_family = strdup ((char *) other->font_family);

    state->viewport_width = other->viewport_width;
    state->viewport_height = other->viewport_height;

    if (other->dash) {
	state->dash = malloc (state->num_dashes * sizeof(double));
	if (state->dash == NULL)
	    return XSVG_STATUS_NO_MEMORY;
	memcpy (state->dash, other->dash, state->num_dashes * sizeof(double));
    }

    return XSVG_STATUS_SUCCESS;
}

xsvg_status_t
_xsvg_state_deinit (xsvg_state_t *state)
{
    if (state->child_surface) {
	cairo_surface_destroy(state->child_surface);
	state->child_surface = NULL;
    }
    
    if (state->font_family) {
	free (state->font_family);
	state->font_family = NULL;
    }

    if (state->dash) {
	free (state->dash);
	state->dash = NULL;
    }

    state->next = NULL;

    return XSVG_STATUS_SUCCESS;
}

xsvg_status_t
_xsvg_state_destroy (xsvg_state_t *state)
{
    _xsvg_state_deinit (state);

    free (state);

    return XSVG_STATUS_SUCCESS;
}

xsvg_state_t *
_xsvg_state_push (xsvg_state_t *state)
{
    xsvg_state_t *new;

    _xsvg_state_create (&new);
    if (new == NULL)
	return NULL;

    _xsvg_state_init_copy (new, state);

    new->next = state;

    return new;
}

xsvg_state_t *
_xsvg_state_pop (xsvg_state_t *state)
{
    xsvg_state_t *next;

    if (state == NULL)
	return NULL;

    next = state->next;

    _xsvg_state_destroy (state);

    return next;
}
